#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import json
import glob
import shutil
import tarfile
import subprocess
from datetime import datetime
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.dialog import dialog
from py_tools.logger import logger
from py_tools.dialog.menu import Menu, MenuItem

log = logger.Logger()

class Dump(object):
    def __init__(self):
        self.d = dialog.Dialog()
        self.d.add_persistent_args(("--backtitle", "Создание/восстановление дампа БД"))
        self.dumpPath = "/linuxcash/cash/data/db-dump"
        self.flashRootPath = "/linuxcash/auto_storage"
        self.mysqlData = "/linuxcash/cash/data/mysql"
        self.mysqlIsRunning = True

    def showWait(self, message):
        log.info(message)
        self.d.infobox(message, height=5, width=40)

    def flashMounted(self):
        result = subprocess.call("mountpoint %s" % self.flashRootPath, shell=True) == 0
        if result:
            log.info("Обнаружен flash накопитель")
        return result

    def getTmpPath(self):
        tmpPath = "/tmp/database-dump"
        if os.path.exists(tmpPath):
            shutil.rmtree(tmpPath)
        os.mkdir(tmpPath)
        return tmpPath

    def getCashReg(self):
        result = {}
        cashregFile = "/linuxcash/cash/data/cash.reg"
        with open(cashregFile) as datafile:
            result = json.load(datafile)
        return result

    def createDump(self):
        try:
            log.info("Выбрано создание дампа")
            choices = [('1', 'Справочники', 1),
                    ('2', 'Продажи', 1)]
            code, tags = self.d.checklist("Выбор БД", height=9, width=0, list_height=2, choices=choices)

            if code != self.d.DIALOG_OK or not tags:
                log.warn("Отменено пользователем")
                return

            tmpPath = self.getTmpPath()
            tmpTarFile = "/tmp/database_dump.tgz"
            if os.path.exists(tmpTarFile):
                os.remove(tmpTarFile)
            log.info("Временный файл архива %s" % tmpTarFile)
            tar = tarfile.open(tmpTarFile, "w:gz")

            metaData = {}
            if '1' in tags:
                self.showWait("Создание дампа справочников ...")
                sqlFile = "%s/dictionaries.sql" % tmpPath
                normalDump = subprocess.call("mysqldump -c dictionaries > %s" % sqlFile, shell=True) == 0
                if normalDump:
                    tar.add(sqlFile, 'dictionaries.sql')
                else:
                    log.error("Ошибка создания дампа при помощи mysqldump, выполняется копирование файлов ...")
                    tar.add(os.path.join(self.mysqlData, "dictionaries"), 'dictionaries')
                dictMetaData = {}
                dictMetaData["dumpType"] = "sql" if normalDump else "files"
                metaData["dictionaries"] = dictMetaData
                log.info("Дамп справочников успешно создан")

            if '2' in tags:
                self.showWait("Создание дампа продаж ...")
                sqlFile = "%s/documents.sql" % tmpPath
                normalDump = subprocess.call("mysqldump -c documents > %s" % sqlFile, shell=True) == 0
                if normalDump:
                    tar.add(sqlFile, 'documents.sql')
                else:
                    log.error("Ошибка создания дампа при помощи mysqldump, выполняется копирование файлов ...")
                    tar.add(os.path.join(self.mysqlData, "documents"), 'documents')
                    tar.add(os.path.join(self.mysqlData, "ibdata1"), 'ibdata1')
                docMetaData = {}
                docMetaData["dumpType"] = "sql" if normalDump else "files"
                metaData["documents"] = docMetaData
                log.info("Дамп продаж успешно создан")

            curTime = datetime.now()
            metaData["time"] = curTime.strftime('%Y-%m-%d %H:%M:%S')
            metadataFile = os.path.join(tmpPath, 'metadata.json')
            with open(metadataFile, 'w') as outfile:
                json.dump(metaData, outfile)

            tar.add(metadataFile, 'metadata.json')
            tar.close()
            log.info("Архив успешно создан")

            destPath = self.dumpPath
            toFlash = False
            if self.flashMounted() and \
                    self.d.yesno("Сохранить дамп на flash накопитель?", height=6, width=0) == self.d.DIALOG_OK:
                toFlash = True
                destPath = self.flashRootPath

            self.showWait("Перемещение архива в %s ..." % destPath)
            if not os.path.exists(destPath):
                os.makedirs(destPath)
            cashReg = self.getCashReg()
            fileName = 'dump_%s_%s_%s.tgz' % (str(cashReg['shopCode']), str(cashReg['cashCode']),
                    curTime.strftime("%Y%m%d_%H%M%S"))
            destDumpPath = os.path.join(destPath, fileName)
            shutil.move(tmpTarFile, destDumpPath)
            if toFlash:
                subprocess.call("sync", shell=True)
            log.info("Дамп успешно создан в %s" % destDumpPath)
            log.info("Перемещение успешно завершено")

            self.d.msgbox("Дамп успешно создан в %s" % destDumpPath, 0, 0)

        except Exception, e:
            log.error("Ошибка создания дампа: %s" % e)
            self.d.msgbox("Ошибка создания дампа", 0, 0)

    def call(self, command, check=True):
        FNULL = open(os.devnull, 'w')
        if subprocess.call(command, shell=True, stdout=FNULL, stderr=subprocess.STDOUT) != 0 and check:
            raise Exception("Ошибка выполнения команды %s" % command)

    def stopMysql(self):
        if self.mysqlIsRunning:
            self.showWait("Остновка сервиса mysql ...")
            self.call("service mysql stop", False)
            self.mysqlIsRunning = False

    def startMysql(self):
        if not self.mysqlIsRunning:
            self.showWait("Запуск сервиса mysql ...")
            self.call("service mysql start", False)
            self.mysqlIsRunning = True

    def extractDir(self, tar, path, dest):
        items = [item for item in tar.getmembers() if item.name.startswith(path + '/')]
        tar.extract(path, dest)
        tar.extractall(path=dest, members=items)

    def restoreDump(self):
        try:
            log.info("Выбрано восстановление из дампа")
            srcPath = self.dumpPath
            fromFlash = False
            if self.flashMounted() and \
                    self.d.yesno("Восстановить дамп с flash накопителя?", height=6, width=40) == self.d.DIALOG_OK:
                fromFlash = True
                srcPath = self.flashRootPath

            dumpFile = self.chooseFile(srcPath)
            if not dumpFile:
                log.warn("Отменено пользователем")
                return
            log.info("Выбран файл %s" % dumpFile)

            tmpPath = self.getTmpPath()
            self.showWait("Распаковка архива ...")
            tar = tarfile.open(dumpFile, "r:gz")
            tar.extract('metadata.json', tmpPath)

            metaData = {}
            with open(os.path.join(tmpPath, 'metadata.json')) as datafile:
                metaData = json.load(datafile)

            showText = u"Дата создания: %s\nВосстановить:" % metaData["time"]
            choices = []
            choices.append(('1', 'Справочники', 1)) if 'dictionaries' in metaData else None
            choices.append(('2', 'Продажи', 1)) if 'documents' in metaData else None

            code, tags = self.d.checklist(showText, title="Подтверждение восстановления",
                    height=10, width=40, list_height=2, choices=choices)

            if code != self.d.DIALOG_OK or not tags:
                log.warn("Отменено пользователем")
                return

            if '1' in tags:
                self.showWait("Восстановление справочников")
                dictMetaData = metaData['dictionaries']
                if dictMetaData['dumpType'] == 'files':
                    log.info("Восстановление из файлов")
                    self.stopMysql()

                    log.info("Удаление файлов")
                    dictTablesPath = os.path.join(self.mysqlData, "dictionaries")
                    if os.path.exists(dictTablesPath):
                        shutil.rmtree(dictTablesPath)

                    log.info("Копирование файлов")
                    self.extractDir(tar, 'dictionaries', self.mysqlData)
                else:
                    log.info("Восстановление из sql дампа")
                    tar.extract('dictionaries.sql', tmpPath)
                    sqlFile = "%s/dictionaries.sql" % tmpPath
                    self.call("mysql -u root dictionaries < %s" % sqlFile)
                log.info("Справочники восстановлены успешно")

            if '2' in tags:
                self.showWait("Восстановление продаж")
                docMetaData = metaData['documents']
                if docMetaData['dumpType'] == 'files':
                    log.info("Восстановление из файлов")
                    self.stopMysql()

                    log.info("Удаление файлов")
                    docTablesPath = os.path.join(self.mysqlData, "documents")
                    if os.path.exists(docTablesPath):
                        shutil.rmtree(docTablesPath)
                    ibdataPath = os.path.join(self.mysqlData, "ibdata1")
                    if os.path.exists(ibdataPath):
                        os.remove(ibdataPath)

                    log.info("Копирование файлов")
                    self.extractDir(tar, 'documents', self.mysqlData)
                    tar.extract('ibdata1', self.mysqlData)
                else:
                    log.info("Восстановление из sql дампа")
                    tar.extract('documents.sql', tmpPath)
                    sqlFile = "%s/documents.sql" % tmpPath
                    self.call("mysql -u root documents < %s" % sqlFile)
                log.info("Продажи восстановлены успешно")
            tar.close()
            self.startMysql()
            self.d.msgbox("Восстановление дампа успешно завершено", 0, 0)
            log.info("Восстановление дампа успешно завершено")

        except Exception, e:
            log.error("Ошибка восстановления из дампа: %s" % e)
            self.d.msgbox("Ошибка восстановления из дампа", 0, 0)

    def chooseFile(self, path):
        log.info("Выбор файла дампа из директории %s" % path)
        dumpFiles = glob.glob("%s/dump*.tgz" % path)
        choices = []
        for i in range(len(dumpFiles)):
            choices.append((str(i + 1), os.path.basename(dumpFiles[i])))
        code, tag = self.d.menu("Выберите файл", width=50, height=16, menu_height=10, choices=choices)
        if code == self.d.DIALOG_OK:
            return dumpFiles[int(tag) - 1]
        else:
            return None

    def run(self):
        log.info("Запуск утилиты создания/восстановления дампа")
        items = [MenuItem("Создать дамп", self.createDump),
                 MenuItem("Восстановить из дампа", self.restoreDump)]
        mainMenu = Menu(self.d, items)
        mainMenu.run()
        log.info("Выход из утилиты создания/восстановления дампа")

if __name__ == "__main__":
    dump = Dump()
    try:
        dump.run()
    except KeyboardInterrupt:
        pass
